# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sorceress']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'sorceress',
    'version': '1.8.5',
    'description': 'Python package for creating optical illusions',
    'long_description': '# sorceress 1.8\n\n[![PyPI version](https://badge.fury.io/py/sorceress.svg)](https://badge.fury.io/py/sorceress) [![Jekyll site CI](https://github.com/altunenes/sorceress/actions/workflows/jekyll.yml/badge.svg)](https://github.com/altunenes/sorceress/actions/workflows/jekyll.yml)\n[![Downloads](https://pepy.tech/badge/sorceress)](https://pepy.tech/project/sorceress)\n\n### Purpose of package\n\nThis package\'s purpose is to create optical illusions in a simple way. The package is written in Python. however, the repo also includes JavaScript.  In [explanations](https://altunenes.github.io/sorceress/explanations%20of%20illusions/) section, you can find the explanations of the illusions.\n\nThis package contains a collection of well-known visual illusions, as well as some lesser-known ones that are just as fascinating.\n\nBy using the code provided in the package, you can easily recreate the illusions and see how they work for yourself.\nNot only can you reproduce the illusions, but you can also experiment with them and modify the code to create your own variations.\n\nWith this package, you can dive deep into the world of visual illusions and learn more about how the human brain processes visual information.\n\nWhether you\'re a researcher, a student, or just someone who loves illusions, this package has something for you. So if you\'re interested in exploring the mysteries of the human visual system, give it a try!\n\nMore importantly, we shouldn\'t take optical illusions as just fun. Optical illusions are visual phenomena that occur when our brain interprets information from our eyes in a way that is different from reality. These illusions can reveal a lot about how our brain works and how it processes visual information, which can help us understand the underlying mechanisms of human vision. Most of the optical illusions in this package are seriously researched in the neuroscience literature. And I must say that the "causation" of the most of effects is still debated in the literature according to my humble knowledge. Optical illusions are researching not only in human vision but in other animals. So we can get a lot of insights from an evolutionary perspective.\n\nThe difficulty of creating an optical illusion with code will depend on the specific illusion you\'re trying to create and your experience with coding. Some optical illusions may be relatively simple to implement, while others may be more complex and require a deeper understanding of graphics programming and the underlying principles of perception.To create an effective optical illusion, you (also me) need to understand the principles of perception and be able to use code to manipulate the visual information in a way that tricks the brain into seeing something that isn\'t actually there. This requires a deep understanding of the underlying mechanisms of perception and a lot of experimentation and trial and error to find the right combination of visual elements that creates the desired illusion.\n\nAnother challenge is that creating optical illusions often requires a high level of precision and control over the visual elements in an image. This can be difficult to achieve with code, especially if you\'re working with complex or dynamic images that need to be updated in real time.\n\nOverall, creating optical illusions with code can be a challenging but rewarding task that requires a combination of technical skills, creativity, and an understanding of the underlying principles of perception.\n\nFor all optical illusions check this documentation: [altunenes.github.io/sorceress/](https://altunenes.github.io/sorceress/)\n\n### Getting started🚀️\n\nThe package can be found on PyPI hence you can install it via pip.\n\n```\npip install sorceress\n```\n\n```\n#importing\nfrom sorceress import sorceress\n```\n\n### Features\n\n[For the API, click here](https://altunenes.github.io/sorceress/api_reference/)\n\n+ Illusions in Python\n  - chromatic\n  - dotill\n  - realtimegrid\n  - addlines\n  - eyecolour\n  - dakinPex\n  - bruno\n  - dolboeuf\n  - kanizsa\n  - tAki2001\n  - cafewall\n  - ccob\n  - ebbinghaus\n  - whiteill\n  - enigma\n  - blackhole\n  - colorgrids\n  - munker\n  - munker2\n  - grids2\n  - pareidolia\n  - spirals\n  - color negative\n\n **Illusions in JavaScript**\n\n  - footsteps\n  - thelilac\n  - EyeMovements\n  - spatialmotion\n  - Motion Induced Blindness (2 versions)\n  - Depth Perception\n  - Speed of Rotation\n  - hypnotic circle\n  - Bright Illusion\n  - Length Illusion\n  - Crazy Sine Waves\n  - Crazy EEG-Fourier\n  \n## Examples\n\n[In this page](https://altunenes.github.io/sorceress/explanations%20of%20illusions/), you can find all illusions, explanations, code, and how to use it. I show just a few examples on this page.\n\n```\nfrom sorceress import sorceress\nsorceress.chromatic("myimage.jpg",circle=False, method="CMCCAT2000", gif=True, Gifduration=7)\nsorceress.addlines("myimage.png",linecolour1=(0,255,0),linecolour2=(0,255,255),linecolour3=(255,0,0))\n```\n\nAs you can see above, you can use the package in a very simple way. You will need to install the following packages to use the package. You will not need to import them, the package will do it for you.\n\n```\npip install opencv-python\npip install numpy\npip install matplotlib\npip install imageio\npip install Pillow\npip install colour-science\n```\n\n## Contribution\n\nAny contribution, bug report, or suggestion is always welcome.\n\n##Author\n\n+ Main Maintainer: Enes Altun\n',
    'author': 'altunenes',
    'author_email': 'enesaltun2@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/altunenes/sorceress',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
